/**
 * @file charge.c
 * @author Tanguy Cavagna (tanguy.cavagna@etu.hesge.ch)
 * @brief Chrage library
 * @version 0.1
 * @date 2021-07-12
 *
 * @copyright Copyright (c) 2021
 *
 */
#include "charge.h"
#include "../draw/draw.h"
#include "../utilities/utils.h"
#include "../vec2/vec2.h"

bool compute_e(charge_t c, vec2 p, double eps, vec2 *e) {
    vec2 qP = vec2_sub(p, c.pos);
    double norm = vec2_norm(qP);
    double E = K * c.q / pow(norm, 2);    // E = k * (Q / r^2)
    *e = vec2_mul(vec2_div(qP, norm), E); // e = E * qP / ||qP||
    return (norm > eps);
}

bool compute_total_normalized_e(charge_t *charges, int num_charges, vec2 p,
                                double eps, vec2 *e) {
    vec2 sum;
    vec2 tmp_e;

    for (int i = 0; i < num_charges; i++) {
        if (!compute_e(charges[i], p, eps, &tmp_e))
            return false;
        sum = vec2_add(sum, tmp_e);
    }

    *e = sum;
    return true;
}

bool compute_p_next(charge_t *charges, int num_charges, double eps,
                    double delta, vec2 *p, bool opposed) {
    vec2 v;
    bool can_draw =
        compute_total_normalized_e(charges, num_charges, *p, eps, &v);

    v = vec2_div(vec2_mul(v, delta), vec2_norm(v));

    if (opposed)
        *p = vec2_sub(*p, v); // P + deltax * E / ||E||
    else
        *p = vec2_add(*p, v); // P - deltax * E / ||E||

    return can_draw;
}

bool pos_contrain_in_universe(vec2 pos, double x0, double x1, double y0,
                              double y1) {
    return (pos.x <= x1 && pos.x >= x0 && pos.y <= y1 && pos.y >= y0);
}

bool pos_not_too_close(vec2 p, charge_t *c, int num_charges) {
    for (int i = 0; i < num_charges; i++) {
        double norm = vec2_norm(vec2_sub(p, c[i].pos));
        if (norm < EPS)
            return false;
    }
    return true;
}

bool field_line_segment(struct gfx_context_t *ctxt, charge_t *charges,
                        int num_charges, double dx, double x0, double x1,
                        double y0, double y1, vec2 old, vec2 p, bool side) {
    while (pos_contrain_in_universe(p, x0, x1, y0, y1) &&
           pos_not_too_close(p, charges, num_charges)) {
        coordinates_t old_coord = position_to_coordinates(
            ctxt->width, ctxt->height, x0, x1, y0, y1, old);
        coordinates_t pi_coord = position_to_coordinates(
            ctxt->width, ctxt->height, x0, x1, y0, y1, p);

        gfx_draw_line(ctxt, pi_coord, old_coord, 0xFFFFFF);

        old = p; // Save current point
        compute_p_next(charges, num_charges, EPS, dx, &p, side);
    }

    return true;
}

void draw_field_line(struct gfx_context_t *ctxt, charge_t *charges,
                     int num_charges, double dx, vec2 pos0, double x0,
                     double x1, double y0, double y1) {
    vec2 p = pos0;
    vec2 old = p;

    bool ended = false;

    for (int side = 0; side <= 1; side++) {
        old = pos0; // Reset to start from the same point for both sides
        field_line_segment(ctxt, charges, num_charges, dx, x0, x1, y0, y1, old,
                           p, (bool)side);
    }
}

void draw_charges(struct gfx_context_t *ctxt, charge_t *charges,
                  int num_charges, double x0, double x1, double y0, double y1) {
    // Center coordinates
    coordinates_t c;

    // Charges coordinates
    coordinates_t ch0;
    coordinates_t ch1;

    int chargePadding = CHARGE_RADIUS / 2;

    // Draw charges
    for (int i = 0; i < num_charges; i++) {
        coordinates_t chCoord = position_to_coordinates(
            ctxt->width, ctxt->height, x0, x1, y0, y1, charges[i].pos);

        // Draw sign according to charge value
        if (charges[i].q < 0) {
            gfx_draw_line(
                ctxt,
                coordinates_create(chCoord.row, chCoord.column - chargePadding),
                coordinates_create(chCoord.row, chCoord.column + chargePadding),
                COLOR_BLUE);
        } else {
            gfx_draw_line(
                ctxt,
                coordinates_create(chCoord.row, chCoord.column - chargePadding),
                coordinates_create(chCoord.row, chCoord.column + chargePadding),
                COLOR_RED);
            gfx_draw_line(
                ctxt,
                coordinates_create(chCoord.row - chargePadding, chCoord.column),
                coordinates_create(chCoord.row + chargePadding, chCoord.column),
                COLOR_RED);
        }

        gfx_draw_circle(ctxt, chCoord, CHARGE_RADIUS,
                        charges[i].q < 0 ? COLOR_BLUE : COLOR_RED);
    }
}

void generate_points(vec2 points[], int nb_points) {
    double x = 0;
    double y = 0;

    for (int i = 0; i < nb_points; i++) {
        x = rand_one();
        y = rand_one();

        points[i] = vec2_create(x, y);
    }
}
