/**
 * @file charge.h
 * @author Tanguy Cavagna (tanguy.cavagna@etu.hesge.ch)
 * @brief Charge file header
 * @version 0.1
 * @date 2021-07-12
 *
 * @copyright Copyright (c) 2021
 *
 */
#ifndef _CHARGE_H_
#define _CHARGE_H_

#define K 8.988e9
#define CHARGE_RADIUS 10
#define EPS 0.025

#include "../gfx/gfx.h"
#include "../utilities/utils.h"
#include "../vec2/vec2.h"
#include <stdbool.h>
#include <stdlib.h>

/**
 * @brief Compute E*qP/norm(qP)
 *
 * @param c Parent charge
 * @param p Point on which to compute e
 * @param eps Minimal gap between field and charge
 * @param e Electric field
 * @return bool false norm(qP) < eps
 */
bool compute_e(charge_t c, vec2 p, double eps, vec2 *e);

/**
 * @brief Compute the normilized sum of Ei*qiP/norm(qiP)
 *
 * @param charges All the charges
 * @param num_charges Number of charges in action
 * @param p Current computed point
 * @param eps Minimal gap between field and charge
 * @param e Electric field
 * @return bool false for some qiP, norm(qiP) < eps
 */
bool compute_total_normalized_e(charge_t *charges, int num_charges, vec2 p,
                                double eps, vec2 *e);

/**
 * @brief Compute the next particle
 *
 * @param charges All the charges
 * @param num_charges Number of charges in action
 * @param eps Minimal gap between field and charge
 * @param delta Distance between two points in the field line
 * @param p Current point
 * @param opposed Is the next point towards charge
 * @return true
 * @return false
 */
bool compute_p_next(charge_t *charges, int num_charges, double eps,
                    double delta, vec2 *p, bool opposed);

/**
 * @brief Draw a segment of a field line between two given points
 *
 * @param ctxt Context to draw on
 * @param charges All the charges
 * @param num_charges Number of charges
 * @param dx Distance between next point and current point
 * @param x0 Minimal X
 * @param x1 Maximal X
 * @param y0 Minimal Y
 * @param y1 Maximal Y
 * @param old Old pos
 * @param p Next pos
 * @param side Side of the line
 * @return true
 * @return false
 */
bool field_line_segment(struct gfx_context_t *ctxt, charge_t *charges,
                        int num_charges, double dx, double x0, double x1,
                        double y0, double y1, vec2 old, vec2 p, bool side);

/**
 * @brief Draw a field line
 *
 * @param ctxt Context to draw on
 * @param charges All the charges in action
 * @param num_charges Number of charges
 * @param dx Distance between  current and next point to draw line
 * @param pos0 Line starting point (will draw on both side of the point)
 * @param x0 Minimal X
 * @param x1 Maximal X
 * @param y0 Minimal Y
 * @param y1 Maximal Y
 */
void draw_field_line(struct gfx_context_t *ctxt, charge_t *charges,
                     int num_charges, double dx, vec2 pos0, double x0,
                     double x1, double y0, double y1);

/**
 * @brief Draw all charges
 *        A circle with a minus sign for negative charges
 *        A circle with a plus sign for positive charges
 *
 * @param ctxt Context to draw on
 * @param charges All the charges
 * @param num_charges Number of charges in action
 * @param x0 Minal x of the universe
 * @param x1 Maximal x of the universe
 * @param y0 Minal y of the universe
 * @param y1 Maximal y of the universe
 */
void draw_charges(struct gfx_context_t *ctxt, charge_t *charges,
                  int num_charges, double x0, double x1, double y0, double y1);

/**
 * @brief Generate a given amount of points
 *
 * @param points Points array
 * @param nb_points Number of point to generate
 */
void generate_points(vec2 points[], int nb_points);

#endif // _CHARGE_H_
