/**
 * @file draw.c
 * @author Tanguy Cavagna (tanguy.cavagna@etu.hesge.ch)
 * @brief Drawing library
 * @version 0.1
 * @date 2021-07-12
 *
 * @copyright Copyright (c) 2021
 *
 */

#include "draw.h"
#include <math.h>

/**
 * @brief Create a coordinate
 *
 * @param row_ Row number
 * @param column_ Column number
 * @return coordinates_t
 */
coordinates_t coordinates_create(int row_, int column_) {
    coordinates_t c = {.row = row_, .column = column_};
    return c;
}

/**
 * @brief Bresenham's line algorithme.
 *        Full understanding of the algorithme, but some parts of the
 *        implementation still blury.
 * @see https://en.wikipedia.org/wiki/Bresenham%27s_line_algorithm#All_cases
 *
 * @param ctxt Context to draw on
 * @param p0 Start point
 * @param p1 End point
 * @param color Stroke color
 */
void gfx_draw_line(struct gfx_context_t *ctxt, coordinates_t p0,
                   coordinates_t p1, uint32_t color) {
    int x0 = p0.column, y0 = p0.row;
    int x1 = p1.column, y1 = p1.row;

    // Single pixel line
    if (x0 == x1 && y0 == y1) {
        gfx_putpixel(ctxt, x0, y0, color);
        return;
    }

    int dx = abs(x1 - x0);
    // Increase if x goes left to right and decrease if right to left
    int sx = x0 < x1 ? 1 : -1;
    int dy = -abs(y1 - y0);
    // Increase if y goes up to down and decrease if down to ups
    int sy = y0 < y1 ? 1 : -1;

    int err = dx + dy; // Error value between x and y
    int e2;

    while (true) {
        gfx_putpixel(ctxt, x0, y0, color);

        if (x0 == x1 && y0 == y1)
            break;

        e2 = 2 * err;

        if (e2 >= dy) { // e_xy + e_x > 0
            err += dy;
            x0 += sx;
        }
        if (e2 <= dx) { // e_xy + e_y < 0
            err += dx;
            y0 += sy;
        }
    }
}

/**
 * @brief Andres's circle algorithm
 * @see
 * https://fr.wikipedia.org/wiki/Algorithme_de_tracé_de_cercle_d%27Andres#Algorithme
 *
 * @param ctxt Context to draw on
 * @param c Center
 * @param r Radius
 * @param color Stroke color
 */
void gfx_draw_circle(struct gfx_context_t *ctxt, coordinates_t c, int r,
                     uint32_t color) {
    int xc = c.column;
    int yc = c.row;
    int x = 0;
    int y = r;
    int d = r - 1;

    while (x < y) {
        // Draw octants pixel
        gfx_putpixel(ctxt, xc + x, yc + y, color);
        gfx_putpixel(ctxt, xc + y, yc + x, color);
        gfx_putpixel(ctxt, xc - x, yc + y, color);
        gfx_putpixel(ctxt, xc - y, yc + x, color);
        gfx_putpixel(ctxt, xc + x, yc - y, color);
        gfx_putpixel(ctxt, xc + y, yc - x, color);
        gfx_putpixel(ctxt, xc - x, yc - y, color);
        gfx_putpixel(ctxt, xc - y, yc - x, color);

        // Rotate
        if (d >= 2 * x) {
            x += 1;
            d -= (2 * x) - 1;
        } else if (d < 2 * (r - y)) {
            y -= 1;
            d += (2 * y) - 1;
        } else {
            y -= 1;
            x += 1;
            d += 2 * (y - x - 1);
        }
    }
}
