/**
 * @file main.c
 * @author Tanguy Cavagna (tanguy.cavagna@etu.hesge.ch)
 * @brief Main file for the electric field exercice
 * @version 0.1
 * @date 2021-07-12
 *
 * @copyright Copyright (c) 2021
 *
 */
#include "charge/charge.h"
#include "gfx/gfx.h"
#include <math.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define WINDOW_WIDTH 500
#define WINDOW_HEIGHT 500
#define x0 0 // Minimal x of the universe
#define x1 1 // Maximal x of the universe
#define y0 0 // Minimal y of the universe
#define y1 1 // Maximal y of the universe
#define NB_CHARGES 3
#define NB_POINTS 200
#define DELTA (1 / sqrt(pow(WINDOW_WIDTH, 2) + pow(WINDOW_HEIGHT, 2)))

void redraw_field(struct gfx_context_t *ctxt, int charge_count,
                  vec2 pos[charge_count], double chs[charge_count]) {
    // Generate all the charges
    charge_t charges[charge_count];
    for (int i = 0; i < charge_count; i++)
        charges[i] = charge_create(chs[i], pos[i]);

    draw_charges(ctxt, charges, charge_count, x0, x1, y0, y1);

    // Generate all the points
    vec2 points[NB_POINTS];
    generate_points(points, NB_POINTS);
    for (int i = 0; i < NB_POINTS; i++) {
        coordinates_t c = position_to_coordinates(WINDOW_WIDTH, WINDOW_HEIGHT,
                                                  x0, x1, y0, y1, points[i]);
        draw_field_line(ctxt, charges, charge_count, DELTA, points[i], x0, x1,
                        y0, y1);
    }
}

int main(void) {
    srand(time(NULL));

    int charge_count = NB_CHARGES;
    vec2 pos[30];
    double chs[30];

    for (int i = 0; i < charge_count; i++) {
        chs[i] = rand_one() / (rand_one() * 10 - 5);
        pos[i] = vec2_create(rand_one(), rand_one());
    }

    // GFX initialization
    struct gfx_context_t *ctxt =
        gfx_create("draw", WINDOW_WIDTH, WINDOW_HEIGHT);
    if (!ctxt) {
        fprintf(stderr, "Graphics initialization failed !\n");
        return EXIT_FAILURE;
    }

    redraw_field(ctxt, charge_count, pos, chs);

    bool running = true;
    // GFX Draw loop
    while (running) {
        SDL_Event event = gfx_event();

        switch (event.type) {
        case SDL_KEYDOWN:
            if (event.key.keysym.sym == SDLK_ESCAPE)
                running = false;
            break;

        case SDL_MOUSEBUTTONDOWN:
            if (event.button.button == SDL_BUTTON_LEFT) {
                charge_count += 1;
                gfx_clear(ctxt, 0x000000);
                int m_x, m_y;
                SDL_GetMouseState(&m_x, &m_y);

                pos[charge_count - 1] = vec2_create(
                    (double)m_x / WINDOW_WIDTH, (double)m_y / WINDOW_HEIGHT);
                chs[charge_count - 1] = rand_one() / (rand_one() * 10 - 5);

                redraw_field(ctxt, charge_count, pos, chs);
            }
            break;
        }

        gfx_present(ctxt);
    }

    gfx_destroy(ctxt);

    return EXIT_SUCCESS;
}
